# -*- coding: utf-8 -*-
"""Unit tests for account endpoints"""

import pytest
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import Account

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


@pytest.mark.usefixtures("mockclient")
class TestAccountEndpoints():
    def test_get_account_success(self, mockclient):
        email = "text@example.com"
        seconds = 10
        data = {"email": email, "balance_seconds": seconds}
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_account()

        assert res == Account(email, seconds)
        mockclient.session.get.assert_called_once_with(
            urljoin(RevAiAPIClient.base_url, "account")
        )

    def test_get_account_not_authorized_error(self, mockclient):
        data = {
            "title": "Authorization has been denied for this request",
            "status": 401
        }
        mockclient.session.get.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.get_account()



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev.ai"""

import requests
import json
from .models import Job, Account, Transcript

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class RevAiAPIClient:
    """Client which implements Rev.ai API"""

    # Default version of Rev.ai
    version = "v1"

    # Default address of the API
    base_url = "https://api.rev.ai/revspeech/{v}/".format(v=version)

    def __init__(self, api_key):
        """Constructor

        :param api_key: api key which authorizes all requests and links them to
            your account. Generated on the settings page of your account
            dashboard on rev.ai
        """
        if not api_key:
            raise ValueError("API Key cannot be empty.")
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': 'Bearer {api_key}'.format(api_key=api_key),
            'User-Agent': 'python_sdk'
        })

    def submit_job_url(self, media_url, options):
        """Submit media given a URL for transcription.
        The audio data is downloaded from the URL.

        :param media_url: web location of the media file
        :param options: JobSubmitOptions object for the job
        :returns: raw response data
        """
        url_jobs = urljoin(self.base_url, "jobs")
        payload = {
            'media_url': media_url,
            'metadata': options.metadata
        }
        if options.callback_url:
            payload['callback_url'] = options.callback_url
        response = self.session.post(url_jobs, json=payload)

        return Job.from_json(response.json())

    def submit_job_local_file(self, filename, options):
        """Submit a local file for transcription.
        Note that the content type is inferred if not provided.

        :param filename: path to a local file on disk
        :param options: JobSubmitOptions object for the job
        :returns: raw response data
        """
        url_jobs = urljoin(self.base_url, "jobs")
        payload = {'metadata': options.metadata}
        if options.callback_url:
            payload['callback_url'] = options.callback_url

        with open(filename, 'rb') as f:
            files = {
                'media': (filename, f),
                'options': (None, json.dumps(payload))
            }
            response = self.session.post(url_jobs, files=files)

        return Job.from_json(response.json())

    def get_job_details(self, id_):
        """View information about a specific job.
        The server will respond with the status and creation date.

        :param id_: id of the job to be requested
        :returns: raw response data
        """
        url_jobs_id = urljoin(self.base_url, "jobs/{id_}".format(id_=id_))

        response = self.session.get(url_jobs_id)

        return Job.from_json(response.json())

    def get_transcript_text(self, id_):
        """Get the transcript of a specific job as json.

        :param id_: id of job to be requested
        :returns: transcript data as text
        """
        url_jobs_transcript = urljoin(
            self.base_url,
            "jobs/{id_}/transcript".format(id_=id_)
        )

        response = self.session.get(
            url_jobs_transcript,
            headers={'Accept': "text/plain"}
        )

        return response.text

    def get_transcript_object(self, id_):
        """Get the transcript of a specific job as json.

        :param id_: id of job to be requested
        :returns: transcript data as a python object
        """
        url_jobs_transcript = urljoin(
            self.base_url,
            "jobs/{id_}/transcript".format(id_=id_)
        )

        response = self.session.get(
            url_jobs_transcript,
            headers={'Accept': 'application/{transcript_version}+json'
                     .format(transcript_version="vnd.rev.transcript.v1.0")}
        )

        return Transcript.from_json(response.json())

    def get_account(self):
        """Get account information, such as remaining balance.
        """
        url_account = urljoin(self.base_url, "account")

        response = self.session.get(url_account)

        return Account.from_json(response.json())



# -*- coding: utf-8 -*-
"""Unit tests for RevAiApiClient"""

import pytest
from src.rev_ai.apiclient import RevAiAPIClient


class TestRevAiAPIClient:
    def test_constructor_success(self):
        token = 'key'

        client = RevAiAPIClient(token)

        headers = client.session.headers
        assert headers.get('User-Agent') == 'python_sdk'
        assert headers.get('Authorization') == 'Bearer {}'.format(token)

    @pytest.mark.parametrize("token", [None, ""])
    def test_constructor_invalid_token_produces_error(self, token):
        with pytest.raises(ValueError, match='API Key cannot be empty.'):
            RevAiAPIClient(token)



# -*- coding: utf-8 -*-
"""Job submit options model"""


class JobSubmitOptions:
    def __init__(self, metadata=None, callback_url=None, skip_diarization=False):
        """
        :param metadata: info to associate with the transcription job
        :param callback_url: callback url to invoke on job completion as a webhook
        :param skip_diarization: should rev.ai skip diaization when transcribing this file
        """
        self.metadata = "" if metadata is None else metadata
        self.callback_url = callback_url
        self.skip_diarization = skip_diarization

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False



# -*- coding: utf-8 -*-
"""Unit tests for transcript endpoints"""

import pytest
from src.rev_ai.models import Transcript
from src.rev_ai.apiclient import RevAiAPIClient

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'


@pytest.mark.usefixtures("mockclient")
class TestTranscriptEndpoints():
    def test_get_transcript_test_success(self, mockclient):
        data = "Hello"
        mockclient.session.get.return_value.text = data

        res = mockclient.get_transcript_text(JOB_ID)

        assert res == data
        mockclient.session.get.assert_called_once_with(
            urljoin(
                RevAiAPIClient.base_url,
                'jobs/{id}/transcript'.format(id=JOB_ID)
            ),
            headers={'Accept': 'text/plain'}
        )

    def test_get_transcript_object_success(self, mockclient):
        data = {
            "monologues": [
                {
                    "speaker": 1,
                    "elements": [
                        {
                            "type": "text",
                            "value": "Hello",
                            "ts": 0.75,
                            "ts_end": 1.25,
                            "confidence": 0.85
                        }
                    ]
                }
            ]
        }
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_transcript_object(JOB_ID)

        assert isinstance(res, Transcript)
        assert len(res.monologues) == 1
        assert res.monologues[0].speaker == 1
        assert len(res.monologues[0].elements) == 1
        assert res.monologues[0].elements[0].value == "Hello"
        mockclient.session.get.assert_called_once_with(
            urljoin(
                RevAiAPIClient.base_url,
                'jobs/{id}/transcript'.format(id=JOB_ID)
            ),
            headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        )

    def test_get_transcript_not_authorized_error(self, mockclient):
        data = {
            "title": "Authorization has been denied for this request",
            "status": 401
        }
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_transcript_object(JOB_ID)

        assert isinstance(res, Transcript)
        assert len(res.monologues) == 0
        mockclient.session.get.assert_called_once_with(
            urljoin(
                RevAiAPIClient.base_url,
                'jobs/{id}/transcript'.format(id=JOB_ID)
            ),
            headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        )

    def test_get_transcript_job_not_found_error(self, mockclient):
        data = {
            "type": "https://www.rev.ai/api/v1/errors/job-not-found",
            "title": "could not find job",
            "status": 404
        }
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_transcript_object(JOB_ID)

        assert isinstance(res, Transcript)
        assert len(res.monologues) == 0
        mockclient.session.get.assert_called_once_with(
            urljoin(
                RevAiAPIClient.base_url,
                'jobs/{id}/transcript'.format(id=JOB_ID)
            ),
            headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        )

    def test_get_transcript_invalid_job_state_error(self, mockclient):
        data = {
            "allowed_values": [
                "transcribed"
            ],
            "current_value": "in_progress",
            "type": "https://rev.ai/api/v1/errors/invalid-job-state",
            "title": "Job is in invalid state",
            "detail": "Job is in invalid state to obtain the transcript",
            "status": 409
        }
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_transcript_object(JOB_ID)

        assert isinstance(res, Transcript)
        assert len(res.monologues) == 0
        mockclient.session.get.assert_called_once_with(
            urljoin(
                RevAiAPIClient.base_url,
                'jobs/{id}/transcript'.format(id=JOB_ID)
            ),
            headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        )



# -*- coding: utf-8 -*-
"""Unit tests for job endpoints"""

import json
import pytest
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import Job, JobStatus, JobSubmitOptions

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
METADATA = 'test'
CALLBACK_URL = 'https://callback.com/'
CREATED_ON = '2018-05-05T23:23:22.29Z'
MEDIA_URL = "https://example.com/test.mp3"


@pytest.mark.usefixtures("mockclient")
class TestJobEndpoints():
    def test_get_job_details_success(self, mockclient):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = {'id': JOB_ID, 'status': status, 'created_on': created_on}
        mockclient.session.get.return_value.json.return_value = data

        res = mockclient.get_job_details(JOB_ID)

        assert res == Job(JOB_ID, created_on, JobStatus.TRANSCRIBED)
        mockclient.session.get.assert_called_once_with(
            urljoin(RevAiAPIClient.base_url, "jobs/{id}".format(id=JOB_ID))
        )

    def test_get_job_details_not_authorized_error(self, mockclient):
        data = {
            "title": "Authorization has been denied for this request",
            "status": 401
        }
        mockclient.session.get.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.get_job_details(JOB_ID)

    def test_get_job_details_job_not_found_error(self, mockclient):
        data = {
            "type": "https://www.rev.ai/api/v1/errors/job-not-found",
            "title": "could not find job",
            "status": 404
        }
        mockclient.session.get.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.get_job_details(JOB_ID)

    def test_submit_job_url_success(self, mockclient):
        status = 'in_progress'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'callback_url': CALLBACK_URL
        }
        mockclient.session.post.return_value.json.return_value = data

        res = mockclient.submit_job_url(
            MEDIA_URL,
            JobSubmitOptions(metadata=METADATA, callback_url=CALLBACK_URL)
        )

        assert res == Job(JOB_ID,
                          CREATED_ON,
                          JobStatus.IN_PROGRESS,
                          metadata=METADATA,
                          callback_url=CALLBACK_URL)
        mockclient.session.post.assert_called_once_with(
            urljoin(RevAiAPIClient.base_url, "jobs"),
            json={
                'media_url': MEDIA_URL,
                'callback_url': CALLBACK_URL,
                'metadata': METADATA
            }
        )

    def test_submit_job_local_file_success(self, mocker, mockclient):
        filename = "test.mp3"
        status = 'in_progress'
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            "id": JOB_ID,
            "status": status,
            "created_on": created_on,
            "metadata": METADATA,
            "callback_url": CALLBACK_URL
        }
        mockclient.session.post.return_value.json.return_value = data

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as mock_open:
            res = mockclient.submit_job_local_file(
                filename,
                JobSubmitOptions(metadata=METADATA, callback_url=CALLBACK_URL)
            )

            assert res == Job(JOB_ID,
                              CREATED_ON,
                              JobStatus.IN_PROGRESS,
                              metadata=METADATA,
                              callback_url=CALLBACK_URL)
            mockclient.session.post.assert_called_once_with(
                urljoin(RevAiAPIClient.base_url, "jobs"),
                files={
                    'media': (filename, mock_open),
                    'options': (None,
                                json.dumps({'metadata': METADATA,
                                            'callback_url': CALLBACK_URL}))
                }
            )

    def test_submit_job_bad_request_error(self, mockclient):
        data = {
            "parameter": {
                "media_url": [
                    "The media_url field is required"
                ],
                "type": "https://www.rev.ai/api/v1/errors/invalid-parameters",
                "title": "Your request parameters didn't validate",
                "status": 400
            }
        }
        mockclient.session.post.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.submit_job_url(MEDIA_URL, JobSubmitOptions())

    def test_submit_job_not_authorized_error(self, mockclient):
        data = {
            "title": "Authorization has been denied for this request",
            "status": 401
        }
        mockclient.session.post.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.submit_job_url(MEDIA_URL, JobSubmitOptions())

    def test_submit_job_insufficient_credits_error(self, mockclient):
        data = {
            "title": "You do not have enough credits",
            "type": "https://www.rev.ai/api/v1/errors/out-of-credit",
            "detail": "You have only 60 seconds remaining",
            "current_balance": 60,
            "status": 403
        }
        mockclient.session.post.return_value.json.return_value = data

        with pytest.raises(KeyError):
            mockclient.submit_job_url(MEDIA_URL, JobSubmitOptions())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from glob import glob
from setuptools import setup, find_packages
try:  # for pip >= 10
    from pip._internal.req import parse_requirements
    from pip._internal.download import PipSession
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements
    from pip.download import PipSession
import re
import ast
import os

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('src/rev_ai/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

parsed_requirements = parse_requirements(
    'requirements.txt',
    session=PipSession())
parsed_test_requirements = parse_requirements(
    'requirements_dev.txt',
    session=PipSession())
requirements = [str(ir.req) for ir in parsed_requirements]
test_requirements = [str(tr.req) for tr in parsed_test_requirements]

setup(
    name='rev_ai',
    version=version,
    description="Rev.ai makes speech applications easy to build!",
    long_description=readme + '\n\n' + history,
    author="Kyle Bridburg",
    author_email='kyle@rev.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[os.path.splitext(os.path.basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    license="MIT license",
    keywords='rev_ai',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    setup_requires=["pytest-runner==4.2"],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/revdotcom/revai-python-sdk',
)



# -*- coding: utf-8 -*-
"""Transcript model"""


class Transcript:
    def __init__(self, monologues):
        """
        :param monologues: list of monologues included in output
        """
        self.monologues = monologues

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            [Monologue.from_json(monologue)
                for monologue in json.get("monologues", [])]
        )


class Monologue:
    def __init__(self, speaker, elements):
        """
        :param speaker: speaker identified for this monologue
        :param elements: list of elements spoken in this monologue
        """
        self.speaker = speaker
        self.elements = elements

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json["speaker"],
            [Element.from_json(element) for element in json.get("elements", [])]
        )


class Element:
    def __init__(self, type_, value, timestamp, end_timestamp, confidence):
        """
        :param type_: type of element: text, punct, or unknown
        :param value: value of the element
        :param timestamp: time at which this element starts in the audio
        :param end_timestamp: time at which this element ends in the audio
        :param confidence: confidence in this output
        """
        self.type_ = type_
        self.value = value
        self.timestamp = timestamp
        self.end_timestamp = end_timestamp
        self.confidence = confidence

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json["type"],
            json["value"],
            json.get("ts"),
            json.get("end_ts"),
            json.get("confidence")
        )



# -*- coding: utf-8 -*-
"""Mock RevAiAPIClient for testing purposes"""

import pytest
from src.rev_ai.apiclient import RevAiAPIClient


@pytest.fixture
def mockclient(mocker):
    client = RevAiAPIClient('key')
    client.session.get = mocker.Mock(name="mock_get")
    client.session.post = mocker.Mock(name="mock_post")
    return client
